﻿// @EXPECTED_RESULTS@: CORRECT
//#define OUTPUT_DEBUG

using System;
using System.Collections.Generic;
using ArrayList = System.Collections.Generic.List<CSharpSolutions.Godzilla.xy>;

namespace CSharpSolutions {
	public class Godzilla {
		public const int MIN_L = 3;
		public const int MIN_W = 3;
		public const int MAX_L = 1000;
		public const int MAX_W = 1000;
		public const int MAX_MECH = 100;
		public const char CHAR_MECH = 'M';
		public const char CHAR_GODZILLA = 'G';
		public const char CHAR_STREET = '.';
		public const char CHAR_RESIDENTIAL = 'R';

		public static void DoGodzilla() {
#if CASECOUNT_GIVEN
			//string testcases = Console.In.ReadLine();
			//int cases = int.Parse(testcases);
			//if (cases < 0 || cases > 100) throw new ArgumentOutOfRangeException("Input error: Wrong number of test cases: '" + cases + "'");
			//Console.Out.WriteLine(cases);

			//for (int i = 0; i < cases; i++)
			//	solve();
#else
			while (solve()) ;
			Console.Out.Flush();
			Console.Out.Close();
#endif // CASECOUNT_GIVEN
		}

		static int l, w;
		static char[][] map;
		static Queue<xy> flood;
		static xy godzilla;
		static int round;
		static int residentialSectorsDestroyed;
		static readonly int[] dx = { 0, 1, 0, -1 };
		static readonly int[] dy = { -1, 0, 1, 0 };
		static readonly int[] gdx = { 0, 1, 0, -1 };
		static readonly int[] gdy = { -1, 0, 1, 0 };
#if CASECOUNT_GIVEN
		private static void solve() {
			string dimensions = Console.In.ReadLine();
#else
		private static bool solve() {
			string dimensions = Console.In.ReadLine();
			if (dimensions == null) return false;
#endif
			string[] split = dimensions.Split(' ');
			if (split.Length != 2) throw new ArgumentException("Input error: Expected 2 elements on this line, found " + split.Length + " instead.");

			// A line containing two integers, the length and width of the map of Tokyo in sectors, respectively;
			if (!int.TryParse(split[0], out l))
				throw new ArgumentException("Input error: Could not parse '" + l + "' as int.");
			if (!int.TryParse(split[1], out w))
				throw new ArgumentException("Input error: Could not parse '" + w + "' as int.");

			if (l < MIN_L || l > MAX_L) throw new ArgumentException("Input error: l not within given limits (l = " + l + ")");
			if (w < MIN_W || w > MAX_W) throw new ArgumentException("Input error: w not within given limits (w = " + w + ")");

			// Then w lines, containing l characters, describing the map of Tokyo.
			map = new char[MAX_W][];
			flood = new Queue<xy>();
			int mechsFound = 0;
			bool godzillaFound = false;

			for (int _y = 0; _y < w; _y++) {
				string line = Console.In.ReadLine();
				if (line == null) throw new ArgumentException("Input error: End of input reached while reading map.");
				map[_y] = new char[MAX_L];
				if (line.Length != l) throw new ArgumentException("Input error: The length of map at y = " + _y + " is not equal to l (" + l + "), but is " + line.Length);
				// Look for Godzilla and the Mechs in this line
				for (int i = 0; i < line.Length; i++) {
					map[_y][i] = line[i];
					switch (map[_y][i]) {
						case CHAR_GODZILLA:
							if (godzillaFound) throw new ArgumentException("Input error: Found duplicate Godzilla in input at (" + i + ", " + _y + ").");
							godzillaFound = true;
							godzilla = new xy();
							godzilla.x = i;
							godzilla.y = _y;
							map[_y][i] = '#';
							break;
						case CHAR_MECH:
							++mechsFound;
							flood.Enqueue(new xy { x = i, y = _y });
							map[_y][i] = '*';
							break;
						case CHAR_STREET:
						case CHAR_RESIDENTIAL:
							// Do nothing
							break;
						default:
							throw new ArgumentException("Input error: Invalid character '" + map[_y][i] + "' in input at (" + i + ", " + _y + ").");
					}
				}
			}
			if (!godzillaFound) throw new ArgumentException("Input error: No Godzilla found in input.");
			if (mechsFound == 0) throw new ArgumentException("Input error: No Mech found in input.");
			if (mechsFound > MAX_MECH) throw new ArgumentException("Input error: More Mechs found (" + mechsFound + ") than specified as max (" + MAX_MECH + ").");

			round = 0;
			residentialSectorsDestroyed = 0;
#if OUTPUT_DEBUG
			drawMap();
#endif // OUTPUT_DEBUG

			// Simulate
			while (true) {
				++round;
				doGodzillaTurn();
				doMechTurn();
#if OUTPUT_DEBUG
				drawMap();
#endif // OUTPUT_DEBUG
				if (godzillaInRange()) break;
				if (round > 100000) {
					drawMap();
					throw new ArgumentException("Input error: No solution is found within round limit.");
				}
			}
#if SOLUTION_ROUNDS
			Console.WriteLine(round);
#else
			Console.WriteLine(residentialSectorsDestroyed);
#endif

#if CASECOUNT_GIVEN
			return;
#else
			return true;
#endif
		}

		private static void drawMap() {
			for (int y = 0; y < w; y++) {
				for (int x = 0; x < l; x++) {
					if (godzilla.x == x && godzilla.y == y)
						Console.Write(CHAR_GODZILLA);
					else Console.Write(map[y][x]);
				}
				Console.WriteLine();
			}
			Console.WriteLine();
		}

		private static void doMechTurn() {
			Queue<xy> newQ = new Queue<xy>();
			while (flood.Count > 0) {
				xy s = flood.Dequeue();
				bool pathObstructed = false;
				for (int j = 0; j < 4; j++) {
					int x = s.x + dx[j];
					if (x < 0 || x >= l) continue;
					int y = s.y + dy[j];
					if (y < 0 || y >= w) continue;

					if (map[y][x] == CHAR_STREET || map[y][x] == '#') {
						newQ.Enqueue(new xy { x = x, y = y });
						map[y][x] = '*';
					}
					else if (!pathObstructed && map[y][x] == CHAR_RESIDENTIAL)
						pathObstructed = true;
				}
				if (pathObstructed)
					newQ.Enqueue(s);
			}
			flood = newQ;
		}

		private static bool godzillaInRange() {
			xy s = new xy { x = godzilla.x, y = godzilla.y };
			while (--s.x >= 0 && map[s.y][s.x] != CHAR_RESIDENTIAL) if (map[s.y][s.x] == '*') return true;
			s.x = godzilla.x;
			while (++s.x < l && map[s.y][s.x] != CHAR_RESIDENTIAL) if (map[s.y][s.x] == '*') return true;
			s.x = godzilla.x;
			while (--s.y >= 0 && map[s.y][s.x] != CHAR_RESIDENTIAL) if (map[s.y][s.x] == '*') return true;
			s.y = godzilla.y;
			while (++s.y < w && map[s.y][s.x] != CHAR_RESIDENTIAL) if (map[s.y][s.x] == '*') return true;

			return false;
		}

		private static void doGodzillaTurn() {
			//Godzilla is lacking in brain matter and has a very predictable movement scheme. First of all,
			//Godzilla does not move into a sector that he already visited and will not move into a sector
			//that contains a mech unit. In addition, on each of its turns:

			// It looks for any houses to destroy that are adjacent to him. If the sector adjacent to him
			//is a residential sector, it will move to that sector and destroy the sector. Godzilla looks
			//around him in the order North, East, South, West. So, if there is no residential sector
			//North of him, he will look East to see if there is a residential sector to destroy, etc.
			for (int j = 0; j < 4; j++) {
				int x = godzilla.x + gdx[j];
				if (x < 0 || x >= l) continue;
				int y = godzilla.y + gdy[j];
				if (y < 0 || y >= w) continue;

				if (map[y][x] == CHAR_RESIDENTIAL) {
					godzilla.x = x;
					godzilla.y = y;
					map[godzilla.y][godzilla.x] = '#';
					++residentialSectorsDestroyed;
					return;
				}
			}

			// In the event that there are no residential sectors adjacent to Godzilla, he will try to
			// move North. If that move would move him outside the map, into a sector he already
			// visited, or into a sector that is occupied by a mech unit, he will instead try to move East.
			// If he cannot move East, he will move South. In the case that he cannot move South, he
			// will try to move West instead.
			for (int j = 0; j < 4; j++) {
				int x = godzilla.x + gdx[j];
				if (x < 0 || x >= l) continue;
				int y = godzilla.y + gdy[j];
				if (y < 0 || y >= w) continue;

				if (map[y][x] == CHAR_STREET) {
					godzilla.x = x;
					godzilla.y = y;
					map[godzilla.y][godzilla.x] = '#';
					return;
				}
			}

			// If Godzilla has no move options left, he will instead start roaring and wail his arms,
			// inflicting no damage to Tokyo city.
		}

		public struct xy {
			public int x, y;
		}
	}
}
