﻿// @EXPECTED_RESULTS@: CORRECT
//#define OUTPUT_DEBUG
//  > "C:\Users\b.pat-el\Documents\T.NET\BAPC 2015\Museum\template\testcases\sample.out"

using System;
using System.Collections.Generic;

namespace CSharpSolutions {
	public class MuseumSlow {
		public const int MIN_L = 1;
		public const int MIN_W = 1;
		public const int MAX_L = 100;
		public const int MAX_W = 100;
		public const double MIN_X = .1d;
		public const double MIN_Y = .1d;
		public const double MIN_SW = .1d;
		public const double WALL_D = .1d;

		public static void DoMuseum() {
#if CASECOUNT_GIVEN
			//string testcases = Console.In.ReadLine();
			//int cases = int.Parse(testcases);
			//if (cases < 0 || cases > 100) throw new ArgumentOutOfRangeException("Input error: Wrong number of test cases: '" + cases + "'");
			//Console.Out.WriteLine(cases);

			//for (int i = 0; i < cases; i++)
			//	solve();
#else
			while (solve()) ;
#endif // CASECOUNT_GIVEN
		}

		static uint l, w;
		static double x, y, sw;
		static line[] pedestal;
#if CASECOUNT_GIVEN
		private static void solve() {
			string dimensions = Console.In.ReadLine();
#else
		private static bool solve() {
			string dimensions = Console.In.ReadLine();
			if (dimensions == null) return false;
#endif
			string[] split = dimensions.Split(' ');
			if (split.Length != 2) throw new ArgumentException("Input error: Expected 2 elements on this line, found " + split.Length + " instead.");
			//A line containing two integers l and w, the length and width of the room in meters, respectively.
			if (!uint.TryParse(split[0], out l))
				throw new ArgumentException("Input error: Could not parse '" + l + "' as int.");
			if (!uint.TryParse(split[1], out w))
				throw new ArgumentException("Input error: Could not parse '" + w + "' as int.");

			if (l < MIN_L || l > MAX_L) throw new ArgumentException("Input error: l not within given limits (l = " + l + ")");
			if (w < MIN_W || w > MAX_W) throw new ArgumentException("Input error: w not within given limits (w = " + w + ")");

			// A line containing three floating point numbers x, y and ws, the x-coordinates, y - coordinates and width of the square pedestal all in meters.
			string pedestalIn = Console.In.ReadLine();
			split = pedestalIn.Split(' ');
			if (split.Length != 3) throw new ArgumentException("Input error: Expected 3 elements on this line, found " + split.Length + " instead.");

			if (!double.TryParse(split[0], out x))
				throw new ArgumentException("Input error: Could not parse '" + x + "' as double.");
			if (!double.TryParse(split[1], out y))
				throw new ArgumentException("Input error: Could not parse '" + y + "' as double.");
			if (!double.TryParse(split[2], out sw))
				throw new ArgumentException("Input error: Could not parse '" + sw + "' as double.");

			if (x < MIN_X || x > l - WALL_D) throw new ArgumentException("Input error: x not within given limits (x = " + x + ")");
			if (y < MIN_Y || y > w - WALL_D) throw new ArgumentException("Input error: y not within given limits (y = " + y + ")");
			if (sw < MIN_SW || sw + x > l - WALL_D || sw + y > w - WALL_D) throw new ArgumentException("Input error: pedestal is too large (sw = " + sw + ")");
#if CASECOUNT_GIVEN
			return;
#else
			// Convert pedestal into 4 lines
			pedestal = new line[4];
			pedestal[0] = new line(x, y, x + sw, y); // Top
			pedestal[1] = new line(x + sw, y, x + sw, y + sw); // Right
			pedestal[2] = new line(x, y + sw, x + sw, y + sw); // Bottom
			pedestal[3] = new line(x, y, x, y + sw); // Left

			var result = naiveAndSlow();
			var sanity = triangleCount();
			// Sanity check
			if (result > sanity)
				throw new Exception("Sanity check failed! More triangles were found than possible! " + result + " found, vs " + sanity + " theoretical.");
			Console.WriteLine(result);

			return true;
#endif
		}

		private static uint triangleCount() {
			uint n = 2 * (l + w);

			// Number of triangles is	(n) =     n!
			//							(3) = 3!(n - 3!)
			var pillarsTop = l + 1;
			var pillarsRight = w + 1;
			var pillarsDown = l + 1;
			var pillarsLeft = w + 1;

			uint triangles = (n * (n - 1) * (n - 2)) / 6;
			// Echter, dat geldt alleen voor als de punten willekeurig liggen. maar dat is hier niet het geval
			// Dus, daar moet je de hoeveelheid driehoeken van af halen die niet gevormd kunnen worden doordat ze op één lijn liggen
			triangles -= ((pillarsTop * (pillarsTop - 1) * (pillarsTop - 2)) / 6) * 2; // Top en bottom
			triangles -= ((pillarsLeft * (pillarsLeft - 1) * (pillarsLeft - 2)) / 6) * 2; // Left en right

			return triangles;
		}

		private static int naiveAndSlow() {
			var list = new List<triangle>();

			var pillars = getPillars();
			foreach (point p1 in pillars) {
				foreach (point p2 in pillars) {
					if (p2 == p1) continue;
					if (p1.x != p2.x || p1.y != p2.y) {
						var l = new line(p1.x, p1.y, p2.x, p2.y);
						if (intersectsPedestal(l)) continue;
					}

					foreach (point p3 in pillars) {
						if (p3 == p2 || p3 == p1) continue;
						if (p1.x == p2.x && p2.x == p3.x) continue;
						if (p1.y == p2.y && p2.y == p3.y) continue;

						if (p1.x != p3.x || p1.y != p3.y) {
							var l = new line(p1.x, p1.y, p3.x, p3.y);
							if (intersectsPedestal(l)) continue;
						}

						if (p2.x != p3.x || p2.y != p3.y) {
							var l = new line(p2.x, p2.y, p3.x, p3.y);
							if (intersectsPedestal(l)) continue;
						}

						var t = new triangle(p1, p2, p3);
						if (!list.Contains(t)) list.Add(t);
					}
				}
			}
			return list.Count;
		}

		private static List<point> getPillars() {
			var result = new List<point>();
			foreach (var p in getPillar())
				result.Add(p);
			return result;
		}

		private static IEnumerable<point> getPillar() {
			foreach (point p in getPillarTop())
				yield return p;
			foreach (point p in getPillarRight())
				yield return p;
			foreach (point p in getPillarDown())
				yield return p;
			foreach (point p in getPillarLeft())
				yield return p;
		}

		private static IEnumerable<point> getPillarTop() {
			for (uint i = 0; i <= l; i++)
				yield return new point { x = i, y = 0 };
		}

		private static IEnumerable<point> getPillarRight() {
			for (uint i = 1; i <= w; i++)
				yield return new point { x = l, y = i };
		}

		private static IEnumerable<point> getPillarDown() {
			for (uint i = 0; i <= l - 1; i++)
				yield return new point { x = i, y = w };
		}

		private static IEnumerable<point> getPillarLeft() {
			for (uint i = 1; i <= w - 1; i++)
				yield return new point { x = 0, y = i };
		}

		private static bool intersectsPedestal(line l) {
			return intersects(l, pedestal[0]) || intersects(l, pedestal[1]) || intersects(l, pedestal[2]) || intersects(l, pedestal[3]);
		}

		private static bool intersects(line l1, line l2) {
			double d = (l2.y2 - l2.y1) * (l1.x2 - l1.x1) - (l2.x2 - l2.x1) * (l1.y2 - l1.y1);
			double n_a = (l2.x2 - l2.x1) * (l1.y1 - l2.y1) - (l2.y2 - l2.y1) * (l1.x1 - l2.x1);
			double n_b = (l1.x2 - l1.x1) * (l1.y1 - l2.y1) - (l1.y2 - l1.y1) * (l1.x1 - l2.x1);

			if (Math.Abs(d) <= 0.0001d)
				return false;

			double ua = n_a / d;
			double ub = n_b / d;

			if (ua >= 0d && ua <= 1d && ub >= 0d && ub <= 1d) return true;
			return false;
		}

		public class line {
			public double x1, y1, x2, y2;

			public line(double x1, double y1, double x2, double y2) {
				this.x1 = x1;
				this.y1 = y1;
				this.x2 = x2;
				this.y2 = y2;
			}
		}

		public class point {
			public uint x;
			public uint y;
			public uint n {
				get { return x + y * w * 5; }
			}
			public override bool Equals(object obj) {
				return this == (point)obj;
			}

			public static bool operator ==(point p1, point p2) {
				return p1.x == p2.x && p1.y == p2.y;
			}

			public static bool operator !=(point p1, point p2) {
				return !(p1 == p2);
			}

			public override int GetHashCode() {
				return x.GetHashCode() * 23 + y.GetHashCode();
			}

		}

		public class triangle {
			public List<point> pillars;
			public int hash;

			public triangle() {
				pillars = new List<point>();
			}

			public triangle(point p1, point p2, point p3) {
				pillars = new List<point> {
					p1, p2, p3
				};
				pillars.Sort(delegate(point _1, point _2) { return _1.n.CompareTo(_2.n); });
				hash = (int)(pillars[0].n + pillars[1].n * 1000 + pillars[2].n * 1000000);
			}

			public override bool Equals(object obj) {
				return this == (triangle)obj;
			}

			public static bool operator ==(triangle t1, triangle t2) {
				return t1.pillars.Contains(t2[0]) && t1.pillars.Contains(t2[1]) && t1.pillars.Contains(t2[2]);
			}

			public static bool operator !=(triangle t1, triangle t2) {
				return !(t1 == t2);
			}

			public point this[int i] {
				get {
					return pillars[i];
				}
				set {
					pillars[i] = value;
				}
			}
			public override int GetHashCode() {
				return hash;
			}
		}
	}
}
