// Incorrect solution to problem `Freight Train'
// Solution that assumes that wagon `nextwagon+length' is part of a train
// of `length' wagons starting at `nextwagon'.
//
// Rudy van Vliet, 19 October, 2015

#include<iostream>
#include<cstring>

using namespace std;
const int MaxN = 1000000000;
const int MaxW = 10000;
const int MaxL = 10000;
int fullWagon[MaxW+1];

//*************************************************************************

bool IntegerInRange (const char *variable, int val, int minVal, int maxVal)
{
  if (val>=minVal && val<=maxVal)
    return true;
  else
  { cout << variable << "=" << val << ", but should be in [" << minVal
         << "," << maxVal << "]" << endl;
    return false;
  }

}  // IntegerInRange

//*************************************************************************

// Use a greedy algorithm to check if it is possible to move all wagons
// containing freight to Luxembourg, using only trains with at most length
// wagons (for Luxembourg, trains going back to the Netherlands may be longer).
bool possible (int N, int W, int L, int length)
{ int lLeft,  // number of locomotives left
      nextWagon,  // next wagon that must be moved to either Luxembourg
                  // or the Netherlands
      nextIndex,  // next index in fullWagon that must be considered
                  // (all earlier wagons have been assigned to a locomotive)
      wagon;      // next wagon containing freight (=fullWagon[nextIndex])

  lLeft = L;
  nextWagon = 1;
  nextIndex= 1;
  while (nextWagon <= N && lLeft > 0)
  { if (nextIndex > W)  // only empty wagons left, which may all
                 // be moved back to the Netherlands by a single locomotive
    { nextWagon = N+1; 
      lLeft --;
    }
    else  // at least one wagon left containing freight
      if (N-nextWagon+1<=length)
        // remaining wagons may all be moved to Luxembourg
        // by a single locomotive
      { nextWagon = N+1;
        lLeft --;
      }
      else  // we cannot move all wagons by one locomotive
            // check how many wagons can be moved by next locomotive
      {
        wagon = fullWagon[nextIndex];
        if (wagon > nextWagon+length)
          // direct a train with empty wagons back to the Netherlands,
          // the wagons nextWagon .. wagon-1
        { nextWagon = wagon;
          lLeft --;
        }
        else
          // direct a train with length wagons to Luxembourg
        {   // first check how many wagons containing freight are in this range
          do
          { nextIndex ++;
          } while (nextIndex<=W && fullWagon[nextIndex]<=nextWagon+length-1);

          nextWagon = nextWagon+length;
          lLeft --;
        }
      }  // else: not one locomotive

  }  // while

  if (nextWagon>N)
    return true;
  else
    return false;

}  // possible

//*************************************************************************

// Use binary search to determine the longest train heading to Luxembourg,
// given that we may use at most L locomotives.
// Pre: N >= 1
//      L >= 1
int binSearch (int N, int W, int L)
{ int leftBound, rightBound,
      mid;

  leftBound = 1;
  rightBound = (N+L-1)/L;  // it is certainly possible with L trains with
                           // length N/L (rounded up)
  mid = (leftBound + rightBound)/2;
    // leftBound < rightBound, if and only if mid < rightBound

  while (mid < rightBound)
  {
    if (possible (N, W, L, mid))
      rightBound = mid;
    else
      leftBound = mid+1;

    mid = (leftBound + rightBound)/2;
  }
                  
  return mid;
  
}  // binSearch

//*************************************************************************

int main()
{ int nInst, inst;
  int N, W, L,
      i,
      result;
  bool OK;

  cin >> nInst;
  for (inst=1;inst<=nInst;inst++)
  {
    OK = true;

    cin >> N >> W >> L;
    if (IntegerInRange ("N", N, 1, MaxN) && IntegerInRange ("W", W, 1, MaxW)
     && IntegerInRange ("W", W, 1, N) && IntegerInRange ("L", L, 1, MaxL))
    { for (i=1;i<=W;i++)
      { cin >> fullWagon[i];
        if (!IntegerInRange ("fullWagon[i]", fullWagon[i], 1, N))
          OK = false;
        if (i>1 && !IntegerInRange ("ascending fullWagon[i]", fullWagon[i],
            fullWagon[i-1]+1, N))  // wagon numbers are not ascending
          OK = false;
      }
    }
    else  // incorrect parameters, so read, but do not process remaining input
    { for (i=1;i<=W;i++)
        cin >> fullWagon[0];
      OK = false;
    }

    if (OK)
    { result = binSearch (N, W, L);
      cout << result << endl;
    }

  }

  return 0;
}
